! This file is part of HiggsBounds
!  
!
! XS_HW_SM,XS_HZ_SM,XS_gg_H_SM,XS_bb_H_SM,XS_vbf:_H_SM
! in units of fb
! functions are fitted to data 
! downloaded from http://maltoni.home.cern.ch/maltoni/TeV4LHC/bbh-tev_ed.dat
! on Thursday 10th April 2008
! data has range 100 to 300 GeV
!
! functions XS_tev_bg_Hb_SM,XS_tev_bg_Hb_c1_SM,XS_tev_bg_Hb_c2_SM were generated by Oliver Brein
!
! on Tuesday 22nd March 2011
! functions XS_lhc7_gg_H_SM,XS_lhc7_HW_SM,XS_lhc7_HZ_SM,XS_lhc7_vbf_SM,XS_lhc7_ttH_SM were generated by Tim Stefaniak
!
! Summer 2012
! Functions for 8 TeV added by OS
!
!****************************************************** 

module theory_XS_SM_functions 

! this must be narrower than range of 
! XS_bg_Hb_SM,XS_bg_Hb_c1_SM,XS_bg_Hb_c2_SM ([10:400])
! XS_tev_gg_H_SM (currently [50:350])
! theory_tevSfunctions (currently [50:400])
!have temporarily set tevXS_SM_functions_xmax to 361, even though this is
!outside the range of XS_tev_gg_H_SM, because the very high Mh are only used for
!tev%XS_Hb_c1_SM at the moment, but we need to be careful (we should calculate XS_tev_gg_H_SM for a bigger range
!as soon as possible)
 double precision :: tevXS_SM_functions_xmin=60.0D0
 double precision :: tevXS_SM_functions_xmax=361.0D0

 double precision :: lhc7XS_SM_functions_xmin=79.0D0
 double precision :: lhc7XS_SM_functions_xmax=1001.0D0

! double precision :: lhc8XS_SM_functions_xmin=79.0D0
 double precision :: lhc8XS_SM_functions_xmin=45.0D0
! double precision :: lhc8XS_SM_functions_xmax=1020.0D0
 double precision :: lhc8XS_SM_functions_xmax=1120.0D0

 contains 

!files from OB (his calculations)
!only change I made:
! end->end function
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin15.etamax2.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin20.etamax2.5.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin15.etamax2.5.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin12.etamax5.h"

 !****************************************************** 
 subroutine check_range(coll,Mh,rangeok,badvalue,strict)
 !****************************************************** 
  implicit none
  character(LEN=4), intent(in) :: coll ! coll should be the same as the elements of 
                                       ! collider in S95tables.f90
  double precision, intent(in) :: Mh 
  logical :: rangeok
  double precision :: badvalue
  logical,optional :: strict

  rangeok=.True.
  if( .not.(Mh.gt.0.0D0) )then !in case Mh is NaN
     rangeok=.False.
  else 
     select case(coll)
     case('TEV ')
      if(    Mh.lt.tevXS_SM_functions_xmin)then 
       rangeok=.False.
      elseif(Mh.gt.tevXS_SM_functions_xmax)then
       rangeok=.False.
      endif
     case('LHC7')
      if(    Mh.lt.lhc7XS_SM_functions_xmin)then 
       rangeok=.False.
      elseif(Mh.gt.lhc7XS_SM_functions_xmax)then
       rangeok=.False.
      endif
     case('LHC8')
      if(    Mh.lt.lhc8XS_SM_functions_xmin)then 
       rangeok=.False.
      elseif(Mh.gt.lhc8XS_SM_functions_xmax)then
       rangeok=.False.
      endif
     case default 
        stop 'problem in subroutine check_range'
     end select
  endif 

  badvalue=0.0D0
  if(present(strict))then
   if(strict)then
    badvalue=-1.0D0
    if(.not.rangeok)then
      write(*,*)'Warning: Higgs mass is outside valid range for the SM cross section functions' 
    endif
   endif
  endif

 end subroutine check_range

 !****************************************************** 
 function XS_tev_bg_Hb_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bg_Hb_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_tev_bg_Hb_SM=func

 end function XS_tev_bg_Hb_SM 
 !****************************************************** 
 function XS_tev_bg_Hb_c1_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bg_Hb_c1_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_c1_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_tev_bg_Hb_c1_SM=func
  
 end function XS_tev_bg_Hb_c1_SM 
 !****************************************************** 
 function XS_tev_bg_Hb_c2_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bg_Hb_c2_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_c2_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_tev_bg_Hb_c2_SM= func
 
 end function XS_tev_bg_Hb_c2_SM 
 !****************************************************** 
 function XS_tev_bg_Hb_c3_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bg_Hb_c3_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=1.0D3*SMCS_tev_bg_Hb_c3(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_tev_bg_Hb_c3_SM= func
 
 end function XS_tev_bg_Hb_c3_SM 
 !****************************************************** 
 function XS_tev_bg_Hb_c4_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bg_Hb_c4_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_c4_SM(x) !convert from pb to fb
  else             
    func=badvalue
  endif

  XS_tev_bg_Hb_c4_SM= func

 end function XS_tev_bg_Hb_c4_SM 
 !****************************************************** 
 function XS_tev_HW_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_HW_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
  
  a0 = 5.7514105496046D0
  a0p5 = -0.375021739545092D0
  a1 = 0.0049451487167627D0
  a2 = -3.77008582179264D-06
  
  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_HW_SM=func

 end function XS_tev_HW_SM
 !****************************************************** 
 function XS_tev_HZ_SM(x,strict)
 !****************************************************** 
  implicit none
  double precision :: XS_tev_HZ_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 5.29935340004443D0
  a0p5 = -0.351677660532052D0
  a1 = 0.0047848452802514D0
  a2 = -3.82425969474559D-06 

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_HZ_SM=func

 end function XS_tev_HZ_SM 
 !****************************************************** 
 !function XS_tev_gg_H_SM(x,strict)
 !******************************************************
 ! implicit none
 ! double precision :: XS_tev_gg_H_SM
 ! double precision, intent(in) :: x 
 ! logical :: rangeok
 ! double precision :: func,badvalue
 ! double precision :: a0,a0p5,a1,a2
 ! logical,optional :: strict
 
 ! a0 = 5.59682783597183D0
 ! a0p5 = -0.244216706673437D0
 ! a1 = 0.000365613425058581D0
 ! a2 = 2.66122261164927D-06  
	
 ! call check_range('TEV ',x,rangeok,badvalue,strict)
 ! if(rangeok)then
 !   func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
 ! else
 !   func=badvalue
 ! endif

 ! XS_tev_gg_H_SM=  func

 !end function XS_tev_gg_H_SM
 !****************************************************** 
  function XS_tev_gg_H_SM(x,strict)
 !******************************************************
 !Updated by TS on 31/03/2011, result given in fb
  implicit none
  double precision :: XS_tev_gg_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 2.82844783978179
  a0p5 = -0.238895314316816
  a1 = -0.00244189137753305
  a2 = 7.90070235250398e-06 
	
  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**3.0D0*10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_gg_H_SM=  func

 end function XS_tev_gg_H_SM
 !****************************************************** 
 function XS_tev_gg_H_SM_9713(x,strict)
 !******************************************************
 ! from CDF note 9713, D0 Note 5889
 ! note: need to check valdity outside 100<x<200

  implicit none
  double precision :: XS_tev_gg_H_SM_9713
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 5.85040646662332D0
  a0p5 = -0.289955377784879D0
  a1 = 0.00367931930924978D0
  a2 =  -5.18285806787161D-06 

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if((x.lt.100.0D0).or.(x.gt.200.0D0))then
   !stop 'function XS_gg_H_SM_9713: need to check validity at this MH'
   write(*,*)'WARNING: function XS_tev_gg_H_SM_9713: need to check validity at this MH'
  endif

  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_gg_H_SM_9713= func
  
 end function XS_tev_gg_H_SM_9713
 !****************************************************** 
 function XS_tev_gg_H_SM_9674(x,strict)
 !******************************************************
 ! from CDF note 9674
 ! note: need to check valdity outside 100<x<200

  implicit none
  double precision :: XS_tev_gg_H_SM_9674
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 5.46563745525111D0
  a0p5 = -0.25740914779426D0
  a1 = 0.00426592493420152D0
  a2 = -9.15606785912482D-06

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if((x.lt.100.0D0).or.(x.gt.200.0D0))then
   !stop 'function XS_gg_H_SM_9674: need to check validity at this MH'
   write(*,*)'WARNING: function XS_gg_H_SM_9674: need to check validity at this MH'
  endif

  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_gg_H_SM_9674=  func
 
 end function XS_tev_gg_H_SM_9674
 !****************************************************** 
 function XS_tev_bb_H_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_bb_H_SM
  double precision, intent(in) :: x
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 5.41583328209568D0
  a0p5 = -0.453323023285831D0
  a1 = 0.00514220061294974D0
  a2 = -3.31488355831377D-06
	
  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_bb_H_SM=  func

 end function XS_tev_bb_H_SM 
 !****************************************************** 
 function XS_tev_vbf_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_vbf_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 3.03330688577339D0
  a0p5 = -0.061167148821396D0
  a1 = -0.00424395931917914D0
  a2 = 7.67964289500027D-08
	
  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_vbf_SM= func

 end function XS_tev_vbf_SM
 !****************************************************** 
 function XS_tev_ttH_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_tev_ttH_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 	
  a0 = 1.23241191811843D0
  a0p5 = 0.188533306634442D0
  a1 = -0.0240486573108739D0
  a2 = 1.90098085776576D-05

  call check_range('TEV ',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_tev_ttH_SM=  func

 end function XS_tev_ttH_SM
 !******************************************************
 function LEPHiggsstrahlung_XS_SM(sqrts, MH)
 !using eq.4.18 in arXiv:hep-ph/0503172 (Abdelhak Djouadi)
 !*****************************************************
 use usefulbits, only : MZ,MW,GF,pi
  implicit none
  double precision :: LEPHiggsstrahlung_XS_SM
  double precision, intent(in) :: sqrts, MH
  double precision :: ae,ve !Z charges of the electron
  double precision :: SW2
  double precision :: lambda !two-particle phase space function
  double precision :: coeff
  double precision :: s,MW2,MZ2,MH2

  s   = sqrts**2.0D0
  MH2 = MH**2.0D0
  MW2 = MW**2.0D0
  MZ2 = MZ**2.0D0
  SW2 = 1.0D0-MW2/MZ2

  ae=-1.0D0
  ve=-1.0D0+4.0D0*SW2

  lambda=(1.0D0 - MH2/s - MZ2/s )**2.0D0 &
        & - 4.0D0*MH2*MZ2/s**2.0D0

  coeff=GF**2.0D0*MZ**4.0D0/(96.0D0*pi) * (ae**2.0D0 + ve**2.0D0)

  LEPHiggsstrahlung_XS_SM= coeff /s *sqrt(lambda) &
           & * ( lambda + 12.0D0 * MZ2/s ) &
           & / ( 1.0d0  -  MZ2/s  )**2.0D0

 end function LEPHiggsstrahlung_XS_SM

 !******************************************************
 ! FUNCTIONS FOR LHC AT 7 TEV
 ! nb: These functions are in units of pb
 !******************************************************
 !******************************************************
 function XS_lhc7_gg_H_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_lhc7_gg_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2,b1,b2,b3,b4,b5,c1,c2,c3,c4,c5
  logical,optional :: strict
 
  a0 = 2.78261222779432D0
  a0p5 = -0.392196551818852D0
  a1 = 0.0117978237757204D0
  a2 = -4.17151874265905D-06
  b1 = -6496.34228398077D0
  b2 = 174.334182737679D0
  b3 = 96.344419701078D0
  b4 = 351.731389836777D0
  b5 = 41.8042207045728D0
  c1 = 1378.60118731267D0
  c2 = 331.890749316436D0
  c3 = 427.044291881834D0
  c4 = 395.248606977478D0
  c5 = -31.7761947614136D0
		
  call check_range('LHC7',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0 &
	& +b1*exp(-(x-b2)**2.0D0/(b3**2.0D0))/((x-b4)**2.0D0+b5**2.0D0) &
	& +c1*exp(-(x-c2)**2.0D0/(c3**2.0D0))/((x-c4)+c5**2.0D0))
  else
    func=badvalue
  endif

  XS_lhc7_gg_H_SM=  func
 
 end function XS_lhc7_gg_H_SM
 
 !******************************************************
 function XS_lhc7_bb_H_SM(x,strict)
 !******************************************************
 !* fit to single Higgs production via b b-bar annihilation 
 !* for a 7 TeV pp collider (NNLO QCD).
 !* Numbers generated with bbh@nnlo v. 1.3 (PDF: MSTW 2008 NNLO) 
 !* with input parameters according to arXiv:1101.0593 [hep-ph].
 !******************************************************
 !* lhc7_cs_bbH_SM : CS in pikobarn
 !* x : Higgs mass in GeV
 !* fit: valid in range [70:1020], deviations from data table below 0.03%
 !* 28/4/2011, Oliver Brein (revised: 29/4/2011)
 !******************************
  implicit none
  double precision :: XS_lhc7_bb_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict
  double precision :: i1,j1,k1,a1,b1,c1,d1,e1,f1,g1,h1

  i1 = 118393.552970347d0
  j1 = -8500.14886364116d0
  k1 = 263.791637006274d0
  a1 = -0.686328023297912d0
  b1 = -0.0263560546043822d0
  c1 = 4.25891594291875d-05
  d1 = -6.72674484878268d-08
  e1 = 7.68476123865973d-11
  f1 = -5.76921139630906d-14
  g1 = 2.52072437882796d-17
  h1 = -4.83311237805624d-21

  call check_range('LHC7',x,rangeok,badvalue,strict)
  if(rangeok)then       
    func=exp( i1/x**3+j1/x**2+k1/x+a1+b1*x+c1*x**2 &
          &   +d1*x**3+e1*x**4+f1*x**5+g1*x**6+h1*x**7  )
  else
    func=badvalue
  endif

  XS_lhc7_bb_H_SM=func
 
 end function XS_lhc7_bb_H_SM
 !****************************************************** 
 function XS_lhc7_HW_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_lhc7_HW_SM
  double precision, intent(in) :: x 
  double precision :: xtemp
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
  
  a0 = 3.47925709357706D0
  a0p5 = -0.405519201979233D0
  a1 = 0.006675415284311D0
  a2 = -1.66890554788541D-06
  
  if(x.gt.305.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

  call check_range('LHC7',xtemp,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_lhc7_HW_SM=func

 end function XS_lhc7_HW_SM
 !****************************************************** 
 function XS_lhc7_HZ_SM(x,strict)
 !****************************************************** 
  implicit none
  double precision :: XS_lhc7_HZ_SM
  double precision, intent(in) :: x 
  double precision :: xtemp
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 2.98356567605733D0
  a0p5 = -0.381566481668429D0
  a1 = 0.00666517818082496D0
  a2 = -3.40355207948873D-06

  if(x.gt.305.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

  call check_range('LHC7',xtemp,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_lhc7_HZ_SM=func

 end function XS_lhc7_HZ_SM 
 !****************************************************** 
 function XS_lhc7_vbf_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_lhc7_vbf_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 
  a0 = 1.05101126849124D0
  a0p5 = -0.0781389982071239D0
  a1 = -0.000788771622392916D0
  a2 = 2.55014215205304D-07
	
  call check_range('LHC7',x,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_lhc7_vbf_SM= func

 end function XS_lhc7_vbf_SM
 !****************************************************** 
 function XS_lhc7_ttH_SM(x,strict)
 !******************************************************
  implicit none
  double precision :: XS_lhc7_ttH_SM
  double precision, intent(in) :: x 
  double precision :: xtemp
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  logical,optional :: strict
 	
  a0 = 1.30235892285854D0
  a0p5 = -0.160563741042278D0
  a1 = -0.00582434890060432D0
  a2 = 1.00116051386356D-06

  if(x.gt.305.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

  call check_range('LHC7',xtemp,rangeok,badvalue,strict)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_lhc7_ttH_SM=  func

 end function XS_lhc7_ttH_SM
 !******************************************************


 !******************************************************
 ! FUNCTIONS FOR LHC AT 8 TEV
 ! nb: These functions are in units of pb
 !******************************************************
 function XS_lhc8_gg_H_SM(x,strict)
 !******************************************************
 ! Update (15/09/2014, TS): New numbers from YR3 in mass
 ! range [80:1000]. Calculated cross section within [60:80]
 ! with Grazzini's online calculator:
 ! http://theory.fi.infn.it/cgi-bin/higgsres.pl
 ! Subtracted 1.5% from result to approximate NLO EW corrections.
 ! Fit function smoothly continues outside valid mass range of
 ! [60:1000], showing no strange artifacts.
 !******************************************************
  implicit none
  double precision :: XS_lhc8_gg_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2,b1,b2,b3,b4,b5,c1,c2,c3,c4,c5
  logical, optional :: strict
		
  call check_range('LHC8',x,rangeok,badvalue,strict)
  
	a0 = 4.80460446867791D0
	a0p5 = -0.561291516848061D0
	a1 = 0.0258452560273618D0
	a2 = -3.07840530611877D-05
	b1 = 375.717055681801D0
	b2 = 241.08386658847D0
	b3 = 35.7144202378D0
	b4 = 310.124404457984D0
	b5 = 10.2091442738781D0  
	c1 = 36478.6666876259D0
	c2 = 497.980740873626D0
	c3 = 468.290091682881D0
	c4 = 361.880403983753D0
	c5 = 103.685654678909D0
   
!  a0   =  3.56707207858078D0
!  a0p5 = -0.25129459795463D0
!  a1   =  0.00433097626519D0
!  a2   = -1.45688652134851D-06
!  b1   = 2770.27394599256D0
!  b2   = 468.519089926562D0
!  b3   = 190.55026272631D0
!  b4   = 379.385111182956D0
!  b5   =  98.6497066533147D0
!  c1   = 494.614733048339D0
!  c2   = 522.274297446605D0
!  c3   = 221.208787555184D0
!  c4   = 381.687044343604D0
!  c5   = -43.5526250947967D0
		
  if(rangeok)then
!    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0 &
!	& +b1*exp(-(x-b2)**2.0D0/(b3**2.0D0))/((x-b4)**2.0D0+b5**2.0D0) &
!	& +c1*exp(-(x-c2)**2.0D0/(c3**2.0D0))/((x-c4)+c5**2.0D0))  
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0 &
  &  +b1*exp(-(x-b2)**2.0D0/(b3**2.0D0))/((x-b4)**2.0D0+b5**2.0D0)) &
  &  +c1*exp(-(x-c2)**2.0D0/(c3**2.0D0))/((x-c4)**2.0D0+c5**2.0D0)

  else
    func=badvalue
  endif

  XS_lhc8_gg_H_SM=  func
 
 end function XS_lhc8_gg_H_SM
 
!******************************************************
 function XS_lhc8_bb_H_SM(x,strict)
 !******************************************************
 !* fit to single Higgs production via b b-bar annihilation 
 !* for a 8 TeV pp collider (NNLO QCD).
 !* Numbers generated with bbh@nnlo v. 1.3 (PDF: MSTW 2008 NNLO) 
 !* with input parameters according to arXiv:1101.0593 [hep-ph].
 !******************************************************
 !* lhc8_cs_bbH_SM : CS in pikobarn
 !* x : Higgs mass in GeV
 !* fit: valid in range [70:1020], deviations from data table below 0.03%
 !* Implemented 5/9/2012 by Oscar Stål (cross sections from Oliver Brein)
 !*******************************************************
 !* Update (16/9/2014, TS): Updated numbers to those of the YR3.
 !* Numbers available for [80, 400] GeV, fit function looks reasonable
 !* for mass range [40, 1100] GeV.
 !******************************************************* 
  implicit none
  double precision :: XS_lhc8_bb_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  logical,optional :: strict
!  double precision :: i1,j1,k1,a1,b1,c1,d1,e1,f1,g1,h1
  double precision :: am1, a0, a0p5, a1, a2

	am1 = 7.28346175088007
	a0 = 2.55313341835946
	a0p5 = -0.34809898410672
	a1 = 0.00377731392689874
	a2 = 1.29486435520287e-07

!        i1 = 179990.036887499D0
!        j1 = -10984.6196406673D0
!        k1 = 303.68366721968D0
!        a1 = -0.86167068733782D0
!        b1 = -0.0236609848244952D0
!        c1 = 3.50207214193607D-05
!        d1 = -5.12839831227492D-08
!        e1 = 5.5011300483384D-11
!        f1 = -3.91883451027439D-14
!        g1 = 1.6391896198688D-17
!        h1 = -3.03114011105266D-21

  call check_range('LHC8',x,rangeok,badvalue,strict)
  if(rangeok)then       
!    func=exp( i1/x**3+j1/x**2+k1/x+a1+b1*x+c1*x**2 &
!          &   +d1*x**3+e1*x**4+f1*x**5+g1*x**6+h1*x**7  )
    func=10.0D0**(am1/x+a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_lhc8_bb_H_SM=func
 
 end function XS_lhc8_bb_H_SM
 !****************************************************** 
  
 !****************************************************** 
 function XS_lhc8_HW_SM(x,strict)
 !******************************************************
 ! Update (16/09/2014, TS): New fit to updated numbers
 ! of the YR3. Underlying numbers in mass range [80,400]
 ! GeV, fit extends reasonably to mass range [20,450] GeV.
 !******************************************************
  implicit none
  double precision :: XS_lhc8_HW_SM
  double precision, intent(in) :: x 
  double precision :: xtemp
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0, a0p5, a1, a2, a3, a4
  logical,optional :: strict
    
!  if(x.gt.305.0D0)then !function is not valid at these masses
  if(x.gt.450.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

!  a0   =  3.43597459323194D0
!  a0p5 = -0.381906179194475D0
!  a1   =  0.00543972777300234D0
!  a2   = -3.66506941121493D-07

	a0 = 3.01235666515895D0
	a0p5 = -0.312688353936888D0
	a1 = 0.00200726253208301D0
	a2 = -2.79580100881394D-06
	a3 = 2.34931925792847D-08
	a4 = -3.27458891365096D-11

  call check_range('LHC8',xtemp,rangeok,badvalue,strict)
  if(rangeok)then
!    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)   
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0+a3*x**3.0D0+a4*x**4.0D0)
  else
    func=badvalue
  endif

  XS_lhc8_HW_SM=func

 end function XS_lhc8_HW_SM
 !****************************************************** 
 function XS_lhc8_HZ_SM(x,strict)
 !******************************************************
 ! Update (16/09/2014, TS): New fit to updated numbers
 ! of the YR3. Underlying numbers in mass range [80,400]
 ! GeV, fit extends reasonably to mass range [20,450] GeV. 
 !****************************************************** 
  implicit none
  double precision :: XS_lhc8_HZ_SM
  double precision, intent(in) :: x 
  double precision :: xtemp
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2,a3,a4
  logical,optional :: strict
 

!  if(x.gt.305.0D0)then !function is not valid at these masses
  if(x.gt.450.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

!  a0   =  3.03485415368548D0
!  a0p5 = -0.376931854913315D0
!  a1   =  0.00660445384856735D0
!  a2   = -3.30648728092992D-06

	a0 = 3.02650891463869D0
	a0p5 = -0.404715124216486D0
	a1 = 0.00945378328194135D0
	a2 = -1.16534141280507D-05
	a3 = 1.301339006803D-08
	a4 = -7.2364967643456D-12

  call check_range('LHC8',xtemp,rangeok,badvalue,strict)
  if(rangeok)then
!    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0+a3*x**3.0D0+a4*x**4.0D0)    
  else
    func=badvalue
  endif

  XS_lhc8_HZ_SM=func

 end function XS_lhc8_HZ_SM 
 
 !****************************************************** 
 function XS_lhc8_vbf_SM(x,strict)
 !******************************************************
 ! Update (16/09/2014): Updated to new numbers from YR3,
 ! for mass range [80.,1000.] GeV. Fit extends reasonable
 ! to mass range [20.,1200.] GeV.
 !******************************************************
  implicit none
  double precision :: XS_lhc8_vbf_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0, a0p5,a1,a2,a3
  logical,optional :: strict

	a0 = 0.997895751135864D0
	a0p5 = -0.0745047859662863D0
	a1 = -0.000594800111789716D0
	a2 = -2.43125249201783D-07
	a3 = 3.10301024559832D-10

 
!  a0   =  1.01415063256363D0 
!  a0p5 = -0.0596858887887809D0
!  a1   = -0.00124307978950821D0
!  a2   =  4.90033650462757D-07
	
  call check_range('LHC8',x,rangeok,badvalue,strict)

  if(rangeok)then
!    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0+a3*x**3.0D0)
    
  else
    func=badvalue
  endif

  XS_lhc8_vbf_SM= func

 end function XS_lhc8_vbf_SM


 !****************************************************** 
 function XS_lhc8_ttH_SM(x,strict)
 !******************************************************
 ! Update (16/09/2014, TS): New fit to updated numbers
 ! of the YR3. Underlying numbers in mass range [80,400]
 ! GeV, fit extends reasonably to mass range [20,450] GeV. 
 !******************************************************
  implicit none
  double precision :: XS_lhc8_ttH_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2,a4
  logical,optional :: strict
  double precision :: xtemp
  
  if(x.gt.450.0D0)then !function is not valid at these masses
   xtemp=1.0D9
  else
   xtemp=x
  endif

	a0 = 1.07128014006971D0
	a0p5 = -0.105671543074652D0
	a1 = -0.00952368203467551D0
	a2 = 1.53282465914447D-05
	a4 = -1.13960404229879D-11

!  a0   = 1.45184703448436D0
!  a0p5 = -0.155124586761319D0
!  a1   = -0.00621310575330019D0
!  a2   = 1.1090649496732D-05
	
  call check_range('LHC8',xtemp,rangeok,badvalue,strict)

  if(rangeok)then
!    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0+a4*x**4.0D0)    
  else
    func=badvalue
  endif

  XS_lhc8_ttH_SM=  func

 end function XS_lhc8_ttH_SM
 !******************************************************


end module theory_XS_SM_functions
